'-----------------------------------------------------------------------
' <copyright file="FileDataView.vb" company="Kanazawa-soft-design,LLC.">
'     Copylight(C) 2018 Kanazawa-soft-design,LLC.
' </copyright>
' <author>金澤 宣明</author>
' <summary></summary>
'-----------------------------------------------------------------------


Partial Class MainForm

	'' -----------------------------------------------------------------------
	''' <summary>
	''' バイナリ表示画面のビューを初期化する.
	''' </summary>
	Private Sub FileData_Init()

		' 垂直スクロールバーを初期化する.
		FileDataVScroll.Value   = 0
		FileDataVScroll.Minimum = 0
		FileDataVScroll.Maximum = 0

		' 水平スクロールバーを初期化する.
		FileDataHScroll.Value   = 0
		FileDataHScroll.Minimum = 0
		FileDataHScroll.Maximum = 0

		' ホイールイベントの追加.
		AddHandler FileData.MouseWheel, New System.Windows.Forms.MouseEventHandler( AddressOf FileData_MouseWheel )

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' ファイルを読み込む.
	''' </summary>
	''' <param name="path">
	''' 読み込むファイル名.
	''' </param>
	''' <returns>
	''' 読み込んだファイル･データを格納したバイト配列.
	''' </returns>
	Private Function ReadFile( path As String ) As Byte()

		' ファイルを読み込みモードでオープンする.
		Using fs As New System.IO.FileStream( path, System.IO.FileMode.Open, System.IO.FileAccess.Read )

			' ファイルのバイト数を取得する.
			Dim size As Integer = fs.Length

			' ファイルサイズが読み込みサイズの上限を超える場合は
			' サイズの上限までしか読み込まない.
			If size > MaxFileSize Then
				size = MaxFileSize
			End If

			' 割り当てられる要素数はインデックス(size) +1となるので -1して確保する.
			Dim buf(size - 1) As Byte

			' ファイルを読み込み用バッファに1回で読み込む.
			fs.Read(buf, 0, buf.Length)
			Return buf

		End Using

	End Function

	'' -----------------------------------------------------------------------
	''' <summary>
	''' ファイルの内容を表示する.
	''' </summary>
	Private Sub ShowFile()

		' 垂直スクロールバーを初期化する.
		FileDataHScroll.Value   = 0
		FileDataVScroll.Value   = 0
		FileDataVScroll.Maximum = (FileBuff.Length / 16)

		' 水平スクロールバーを初期化する.
		FileDataHScroll.Value   = 0
		FileDataHScroll.Minimum = 0
		FileDataHScroll.Maximum = 74

		' 画面に合わせてスクロールバーのつまみ幅を設定する.
		SetScrollBar()

		' バイナリ表示画面を再描画する.
		SplitContainer2.Panel2.Refresh()

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' ファイル･データのバイナリ表示画面を描画する.
	''' </summary>
	''' <param name="sender">
	''' イベント対象のオブジェクト.
	''' </param>
	''' <param name="e">
	''' イベントパラメータ.
	''' </param>
	Private Sub FileData_Paint( sender As Object, e As PaintEventArgs ) Handles FileData.Paint

		Dim rc As Rectangle = e.ClipRectangle
		Dim g  As Graphics  = e.Graphics

		' 画面をクリアする.
		g.FillRectangle( Brushes.White, 0, 0, rc.Width, rc.Height )

		' ファイルデータが無い場合は描画を行わない.
		If FileBuff Is Nothing Then
			Return
		End If

		' フォントを作成する.
		Dim fnt As New Font( "ＭＳ ゴシック", 12 )

		Dim x As Single = 0
		Dim y As Single = 0

		Dim colomuns As Integer = 1
		Using sf As New StringFormat( StringFormat.GenericTypographic )

			' 1文字の幅を取得する.
			Dim fs = g.MeasureString( "0", fnt, rc.Width, sf )

			' 水平方向の書き出し座標を計算する.
			x = -1 * (FileDataHScroll.Value * fs.Width)

			' 1画面に表示可能な文字数を計算する.
			If fs.Width > 0 Then
				colomuns = (rc.Width / fs.Width)
			End If
		End Using

		' 1画面に表示可能な行数を計算する.
		Dim lines As Integer = 1
		If fnt.Height > 0 Then
			lines = ((rc.Height) + (fnt.Height - 1)) / fnt.Height
		End If

		' スケールを描画する.
		y = DrawScale( g, rc, fnt, x, y )

		' ファイルデータを16進数で描画する.
		Dim adr As Integer = FileDataVScroll.Value * 16
		For i As Integer = 1 To lines
			DrawBinaryData( g, rc, fnt, x, y, adr )
			adr += 16
			y   += fnt.Height
		Next

		' フォントを破棄する.
		fnt.Dispose()

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' スケールを描画する.
	''' </summary>
	''' <param name="g">
	''' 描画に使用するグラフィックス.
	''' </param>
	''' <param name="rc">
	''' クライアント領域のサイズ.
	''' </param>
	''' <param name="fnt">
	''' フォント.
	''' </param>
	''' <param name="x">
	''' 描画開始するX座標.
	''' </param>
	''' <param name="y">
	''' 描画開始するY座標.
	''' </param>
	''' <returns>
	''' 次に描画を開始するY座標.
	''' </returns>
	Private Function DrawScale(g As Graphics, rc As Rectangle, fnt As Font, x As Single, y As Single) As Single

		Using sf As New StringFormat(StringFormat.GenericTypographic)

			' アドレスを描画する領域を塗りつぶす.
			Dim adrArea As SizeF = g.MeasureString("000000000", fnt, rc.Width, sf)
			g.FillRectangle( Brushes.Gray, x, 0, adrArea.Width, rc.Height )

			' スケールを描画する.
			g.FillRectangle( Brushes.Gray, 0, 0, rc.Width, fnt.Height )
			g.DrawString("ADDRESS:  +0 +1 +2 +3 +4 +5 +6 +7 +8 +9 +A +B +C +D +E +F ASCII", fnt, Brushes.White, x, y, sf)
			y += fnt.Height

		End Using

		Return y

	End Function

	'' -----------------------------------------------------------------------
	''' <summary>
	''' 1行分のバイナリ･データを描画する.
	''' </summary>
	''' <param name="g">
	''' 描画に使用するグラフィックス.
	''' </param>
	''' <param name="rc">
	''' クライアント領域のサイズ.
	''' </param>
	''' <param name="fnt">
	''' フォント.
	''' </param>
	''' <param name="x">
	''' 描画開始するX座標.
	''' </param>
	''' <param name="y">
	''' 描画開始するY座標.
	''' </param>
	''' <param name="adr">
	''' 描画開始するバイナリデータのアドレス.
	''' </param>
	Private Sub DrawBinaryData( g As Graphics, rc As Rectangle, fnt As Font, x As Single, y As Single, adr As Integer )

		If adr >= FileBuff.Length Then
			Return
		End If

		Using sf As New StringFormat( StringFormat.GenericTypographic )

			Dim fs As SizeF = g.MeasureString( "0", fnt, rc.Width, sf )

			' アドレスを描画する.
			Dim adrText As String = adr.ToString( "X8" )
			g.DrawString( adrText, fnt, Brushes.White, x, y, sf )

			Dim x1 As Single = x + (9 * fs.Width)

			Dim ascii As String = String.Empty
			For i As Integer = 0 To 15
				Dim byteText As String = String.Empty

				If (adr + i) < FileBuff.Length Then
					Dim c As UInteger = FileBuff(adr + i)

					byteText = " " + c.ToString("X2")

					If &H20 <= c And c < &H80 Then
						ascii += Convert.ToChar( c )
					Else
						ascii += "."
					End If

				Else
					byteText = "   "
				End If

				g.DrawString( byteText, fnt, Brushes.Black, x1, y, sf )
				x1 += (3 * fs.Width)
			Next

			Dim asciiText As String = " " + ascii
			g.DrawString( asciiText, fnt, Brushes.Black, x1, y, sf )

		End Using
	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' スクロールバーを設定する.
	''' </summary>
	Private Sub SetScrollBar()

		' クライアント領域のサイズを取得する.
		Dim rc As System.Drawing.Size = FileData.ClientSize

		' グラフィックス･オブジェクトを取得する.
		Dim g As Graphics = FileData.CreateGraphics()

		' フォントを作成する.
		Dim fnt As New Font( "ＭＳ ゴシック", 12 )

		' 1画面に表示可能な行数を計算する.
		Dim lines As Integer = 1
		If fnt.Height > 0 Then
			lines = ((rc.Height) + (fnt.Height - 1)) / fnt.Height
		End If

		Dim colomuns As Integer = 1
		Using sf As New StringFormat( StringFormat.GenericTypographic )

			' 1文字の幅を取得する.
			Dim fs = g.MeasureString( "0", fnt, rc.Width, sf )

			' 1画面に表示可能な文字数を計算する.
			If fs.Width > 0 Then
				colomuns = (rc.Width / fs.Width)
			End If
		End Using

		' スクロールバーのつまみ幅を画面に合わせて再設定する.
		FileDataVScroll.LargeChange = lines    / 2
		FileDataHScroll.LargeChange = colomuns / 2

		' フォントを破棄する.
		fnt.Dispose()

		' グラフィックス･オブジェクトを解放する.
		g.Dispose()

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' バイナリ表示画面の垂直スクロールバーが操作されたときのイベント処理.
	''' </summary>
	''' <param name="sender">
	''' イベント対象のオブジェクト.
	''' </param>
	''' <param name="e">
	''' イベントパラメータ.
	''' </param>
	Private Sub FileDataVScroll_Scroll(sender As Object, e As ScrollEventArgs) Handles FileDataVScroll.Scroll

		' バイナリ表示画面を再描画する.
		SplitContainer2.Panel2.Refresh()

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' バイナリ表示画面の水平スクロールバーが操作されたときのイベント処理.
	''' </summary>
	''' <param name="sender">
	''' イベント対象のオブジェクト.
	''' </param>
	''' <param name="e">
	''' イベントパラメータ.
	''' </param>
	Private Sub FileDataHScroll_Scroll(sender As Object, e As ScrollEventArgs) Handles FileDataHScroll.Scroll

		' バイナリ表示画面を再描画する.
		SplitContainer2.Panel2.Refresh()

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' マウス･ホイールのイベント処理.
	''' </summary>
	''' <param name="sender">
	''' イベント対象のオブジェクト.
	''' </param>
	''' <param name="e">
	''' イベントパラメータ.
	''' </param>
	Private Sub FileData_MouseWheel( sender As Object, e As MouseEventArgs )

		' スクロール量（方向）の表示.
		Dim val As Single = FileDataVScroll.Value - (e.Delta * SystemInformation.MouseWheelScrollLines / 120)

		If  val < FileDataVScroll.Minimum Then
			val = FileDataVScroll.Minimum
		End If

		Dim maximun As Integer = FileDataVScroll.Maximum - (FileDataVScroll.LargeChange - 1)
		If  val > maximun Then
			val = maximun
		End If

		FileDataVScroll.Value = val

		' バイナリ表示画面を再描画する.
		splitContainer2.Panel2.Refresh()

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' ファイルデータ表示ビューがクリックされたときのイベント処理.
	''' </summary>
	''' <param name="sender">
	''' イベントを発生させたオブジェクト.
	''' </param>
	''' <param name="e">
	''' イベント･パラメータ.
	''' </param>
	Private Sub FileData_Click( sender As Object, e As EventArgs ) Handles FileData.Click

		' ピクチャボックスにフォーカスを移す.
		FileData.Focus()

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' ウインドウサイズが変更されたときのイベント処理.
	''' </summary>
	''' <param name="sender">
	''' イベントを発生させたオブジェクト.
	''' </param>
	''' <param name="e">
	''' イベント･パラメータ.
	''' </param>
	Private Sub FileData_SizeChanged( sender As Object, e As EventArgs ) Handles FileData.SizeChanged

		' スクロールバーを設定する.
		SetScrollBar()

	End Sub

End Class
