'-----------------------------------------------------------------------
' <copyright file="FileListView.vb" company="Kanazawa-soft-design,LLC.">
'     Copylight(C) 2018 Kanazawa-soft-design,LLC.
' </copyright>
' <author>金澤 宣明</author>
' <summary></summary>
'-----------------------------------------------------------------------

Partial Class MainForm

	'' -----------------------------------------------------------------------
	''' <summary>
	''' ファイル一覧のビューを初期化する.
	''' </summary>
	Private Sub FileList_Init()

		' ファイル一覧にカラムを追加する.
		FileList.Columns.Add( "ファイル名", 200, HorizontalAlignment.Left  )
		FileList.Columns.Add( "サイズ"	  , 120, HorizontalAlignment.Right )
		FileList.Columns.Add( "更新日時"  , 150, HorizontalAlignment.Left  )

		' ファイル一覧にドライブ名を表示する.
		ShowDriveName()

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' ファイル一覧を更新する.
	''' </summary>
	''' <param name="path">
	''' 一覧表示するディレクトリ名.
	''' </param>
	Private Sub UpdateFileList( path As String )

		' 一覧表示するディレクトリ名を保存する.
		CurrentDir = path

		If CurrentDir.Length > 0 Then
			' 指定されたパスのファイル一覧を生成する.
			ShowFileName( path )
		Else
			' パスが指定されていない場合はドライブ一覧を生成する.
			ShowDriveName()
		End If

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' ファイル一覧にドライブ名の一覧を表示する.
	''' </summary>
	Private Sub ShowDriveName()

		' ファイル一覧の表示を一旦クリアする.
		FileList.Items.Clear()

		' 論理ドライブ名をすべて取得する.
		Dim stDrives As String() = System.IO.Directory.GetLogicalDrives()

		' 取得した論理ドライブ名をすべて表示する.
		For Each drive As String In stDrives

			' ドライブ名をの最後にあるセパレータ("\")を取り除く.
			Dim drvname As String = drive.Substring( 0, drive.Length - 1 )

			' ファイル一覧に生成したファイル情報を追加する.
			FileList.Items.Add( New ListViewItem( New string() { drvname, "", "" }) )
		Next

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' ファイル一覧にファイル名の一覧を表示する.
	''' </summary>
	''' <param name="path">
	''' ファイル一覧を表示するディレクトリの絶対パス.
	''' </param>
	Private Sub ShowFileName( path As String )

		' ファイル一覧の表示を一旦クリアする.
		FileList.Items.Clear()

		' 一つ上のディレクトリに移動する為の「..」をリストの先頭に追加する.
		FileList.Items.Add( New ListViewItem( New string() { "..", "<DIR>", "" }) )

		Try
			' 指定されたディレクトリ直下のサブディレクトリ名のリストを取得する.
			path += "\"
			For Each dir As String In System.IO.Directory.GetDirectories( path )

				' ディレクトリの属性を取得する.
				Dim di As New System.IO.DirectoryInfo( dir )

				' 隠しディレクトリはリストに表示しない.
				If (di.Attributes And System.IO.FileAttributes.Hidden) <> System.IO.FileAttributes.Hidden Then

					' 更新日付を文字列にする.
					Dim dateStr As String  = di.LastWriteTime.ToString( "yyyy/MM/dd hh:mm" )

					' ファイル一覧に生成したディレクトリ情報を追加する.
					FileList.Items.Add( New ListViewItem( New string() { di.Name, "<DIR>", dateStr }) )
				End If
			Next

			' 指定されたディレクトリ直下のファイル名のリストを取得する.
			For Each file As String In System.IO.Directory.GetFiles( path, "*", System.IO.SearchOption.TopDirectoryOnly )

				' ファイル属性を取得する.
				Dim attr As System.IO.FileAttributes = System.IO.File.GetAttributes( file )

				' 隠しファイルはリストに表示しない.
				If (attr And System.IO.FileAttributes.Hidden) <> System.IO.FileAttributes.Hidden Then

					' ファイル名からファイル情報を取得する.
					Dim fi As New System.IO.FileInfo( file )

					' ファイルサイズを右詰の文字列に変換する.
					Dim sizeStr As String = fi.Length.ToString( "#,0" )

					' 更新日付を文字列にする.
					Dim dateStr As String = fi.LastWriteTime.ToString( "yyyy/MM/dd hh:mm" )

					' ファイル一覧に生成したファイル情報を追加する.
					FileList.Items.Add( New ListViewItem( New string() { fi.Name, sizeStr, dateStr }) )
				End If
			Next

		Catch ex As System.Exception

		End Try

	End Sub

	'' -----------------------------------------------------------------------
	''' <summary>
	''' ファイル一覧の項目がダブルクリックされたときの操作イベント.
	''' </summary>
	''' <param name="sender">
	''' イベント対象のオブジェクト.
	''' </param>
	''' <param name="e">
	''' イベントパラメータ.
	''' </param>
	Private Sub FileList_DoubleClick( sender As Object, e As EventArgs ) Handles FileList.DoubleClick

		If FileList.SelectedItems.Count > 0 Then

			' リストビューからファイル名を取り出して、カレントディレクトリ名と連結して
			' 選択されたファイルの絶対パスにする.
			Dim filename As String = FileList.SelectedItems(0).Text
			Dim fullpath As String
			If CurrentDir.Length > 0 Then
				fullpath = CurrentDir + "\" + filename
			Else
				fullpath = filename
			End If

			' ".."の場合、カレントパスの最後のディレクトリ名を取り除く.
			If filename.Equals( ".." ) Then
				Dim pos As Integer = CurrentDir.LastIndexOf( "\" )
				If pos >= 0 Then
					fullpath = CurrentDir.Substring( 0, pos )
				Else
					ExpandDirectory( "" )
					Return
				End If
			End If

			Dim attr As System.IO.FileAttributes = System.IO.File.GetAttributes(fullpath)
			If (attr And System.IO.FileAttributes.Directory) = System.IO.FileAttributes.Directory Then
				' 選択されたファイルがディレクトリの場合、選択されたディレクトリに移動する.
				ExpandDirectory( fullpath )
			Else
				' ファイルを読み込む.
				FileBuff = ReadFile( fullpath )

				' ステータスバーにファイル名を表示する.
				StatusLabel1.Text = fullpath

				' 読み込んだファイルの内容を表示する.
				ShowFile()
			End If
		End If

	End Sub

End Class
